package com.example.android.gpslogger;

import android.app.Service;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.widget.Toast;

public class GpsLogService extends Service {
    public static final String ACTION_LOCATION_CHANGED = "com.example.android.gpslogger.LOCATION_CHANGED";
    private static final int MIN_TIME = 1 * 60000; // 1分
    private static final float MIN_DISTANCE = 100; // 100m
    private static final float ACCURACY_THRESHOLD = 100; // 100m

    private final LocationListener listener = new LocationListener() {
        public void onLocationChanged(Location location) {
            // 十分な精度がとれたら記録する。
            if (location.getAccuracy() < ACCURACY_THRESHOLD)
                updateLocation(location);
        }

        public void onProviderDisabled(String provider) {
        }

        public void onProviderEnabled(String provider) {
        }

        public void onStatusChanged(String provider, int status, Bundle extras) {
        }
    };

    private final LocationListener bounceListener = new LocationListener() {
        public void onLocationChanged(Location location) {
            if (location.getAccuracy() < ACCURACY_THRESHOLD) {
                // 十分な精度がとれたら記録する。
                updateLocation(location);
                manager.removeUpdates(this);
            }
        }

        public void onProviderDisabled(String provider) {
        }

        public void onProviderEnabled(String provider) {
        }

        public void onStatusChanged(String provider, int status, Bundle extras) {
        }
    };

    private LocationManager manager;
    private LocationDatabase dao;

    @Override
    public void onCreate() {
        super.onCreate();
        manager = (LocationManager) getSystemService(LOCATION_SERVICE);
        dao = new LocationDatabase(this);
    }

    @Override
    public void onStart(Intent intent, int startId) {
        handleCommand(intent);
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        handleCommand(intent);
        // START_STICKYをreturnして、システムがサービスを強制終了した後に再始動するように依頼する
        return START_STICKY;
    }

    private void handleCommand(Intent intent) {
        Toast.makeText(this, getString(R.string.start_service), Toast.LENGTH_SHORT).show();
        unregisterListener();
        registerListener(MIN_TIME, MIN_DISTANCE);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Toast.makeText(this, getString(R.string.stop_service), Toast.LENGTH_SHORT).show();
        unregisterListener();
        dao.close();
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    private void registerListener(long minTime, float minDistance) {
        manager.requestLocationUpdates(LocationManager.GPS_PROVIDER, minTime, minDistance, listener);
        manager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, bounceListener);
    }

    private void unregisterListener() {
        manager.removeUpdates(listener);
        manager.removeUpdates(bounceListener);
    }

    private void updateLocation(final Location location) {
        dao.insert(location);
        Intent intent = new Intent(ACTION_LOCATION_CHANGED);
        sendBroadcast(intent);
    }
}