package com.example.android.gpslogger;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.location.Location;

public class LocationDatabase extends SQLiteOpenHelper {

    private static final String[] COLUMNS = {
            "_id",
            "latitude",
            "longitude",
            "time" };

    public LocationDatabase(Context context) {
        super(context, "data", null, 1);
    }

    @Override
    public void onCreate(SQLiteDatabase db) {
        db.execSQL("create table location ("
                + "_id integer primary key autoincrement," 
                + "latitude text not null, " 
                + "longitude text not null, " 
                + "time text not null" + ")");
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        db.execSQL("drop table if exists location");
        onCreate(db);
    }

    public long insert(Location location) {
        ContentValues values = new ContentValues();
        values.put("latitude", Double.toString(location.getLatitude()));
        values.put("longitude", Double.toString(location.getLongitude()));
        values.put("time", Long.toString(location.getTime()));
        SQLiteDatabase db = getWritableDatabase();
        try {
            return db.insert("location", null, values);
        } finally {
            db.close();
        }
    }

    public Cursor fetchAll() {
        SQLiteDatabase db = getReadableDatabase();
        return db.query("location", COLUMNS, null, null, null, null, "time");
    }

    public int deleteAll() {
        SQLiteDatabase db = getWritableDatabase();
        onUpgrade(db, 0, 0);
        try {
            return db.delete("location", null, null);
        } finally {
            db.close();
        }
    }
}
