package com.example.android.gpslogger;

import java.text.SimpleDateFormat;
import java.util.Date;

import android.app.Activity;
import android.app.ListActivity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CursorAdapter;
import android.widget.TextView;

public class MyActivity extends ListActivity {

    // Viewの参照を保持するクラス
    private static final class Holder {
        TextView latitude;
        TextView longitude;
        TextView time;
    }

    private final class MyAdapter extends CursorAdapter {
        // カラム位置をキャッシュする
        final int latitudeIndex;
        final int longitudeIndex;
        final int timeIndex;

        public MyAdapter(Context context, Cursor c) {
            super(context, c);
            // カラム位置を取得してキャッシュしておく
            latitudeIndex = c.getColumnIndex("latitude");
            longitudeIndex = c.getColumnIndex("longitude");
            timeIndex = c.getColumnIndex("time");
        }

        @Override
        public void bindView(final View view, Context context, Cursor cursor) {
            // Holderに保持したViewの参照を通じて、データを設定する
            Holder h = (Holder) view.getTag();
            h.latitude.setText(cursor.getString(latitudeIndex));
            h.longitude.setText(cursor.getString(longitudeIndex));
            h.time.setText(formatTime(cursor.getLong(timeIndex)));
        }

        @Override
        public View newView(Context context, Cursor cursor, ViewGroup parent) {
            LayoutInflater inflater = LayoutInflater.from(self);
            View v = inflater.inflate(R.layout.location, parent, false);
            // パフォーマンス向上のためにViewの参照をHolderに保持しておく
            Holder h = new Holder();
            h.latitude = (TextView) v.findViewById(R.id.latitude);
            h.longitude = (TextView) v.findViewById(R.id.longitude);
            h.time = (TextView) v.findViewById(R.id.time);
            v.setTag(h);
            return v;
        }
    }

    private final BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            adapter.getCursor().requery();
        }
    };

    private final View.OnClickListener startButtonClickListener = new View.OnClickListener() {
        public void onClick(View v) {
            Intent i = new Intent(self, GpsLogService.class);
            startService(i);
        }
    };

    private final View.OnClickListener stopButtonClickListener = new View.OnClickListener() {
        public void onClick(View v) {
            Intent i = new Intent(self, GpsLogService.class);
            stopService(i);
        }
    };

    private final View.OnClickListener clearButtonClickListener = new View.OnClickListener() {
        public void onClick(View v) {
            // Applicationが処理中に応答しなくなるのを防ぐ
            new AsyncTask<Void, Void, Void>() {
                @Override
                protected Void doInBackground(Void... params) {
                    // データの削除をバックグラウンドで実行する
                    // UIスレッドで行うとアプリケーションの応答性が低下する
                    dao.deleteAll();
                    return null;
                }

                @Override
                protected void onPostExecute(Void result) {
                    // データの表示はUIスレッドで行わなければならない
                    adapter.getCursor().requery();
                };
            }.execute();
        }
    };

    private final View.OnClickListener mapButtonClickListener = new View.OnClickListener() {
        public void onClick(View v) {
            Intent i = new Intent(self, MyMapActivity.class);
            startActivity(i);
        }
    };

    private Activity self;
    private volatile CursorAdapter adapter;
    private LocationDatabase dao;

    // フォーマッターをキャッシュします。
    private SimpleDateFormat timeFormatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
    // フォーマッターの引数に指定する日付オブジェクトをキャッシュします。
    private Date date = new Date();

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        self = this;

        findViewById(R.id.button_start).setOnClickListener(startButtonClickListener);
        findViewById(R.id.button_stop).setOnClickListener(stopButtonClickListener);
        findViewById(R.id.button_clear).setOnClickListener(clearButtonClickListener);
        findViewById(R.id.button_map).setOnClickListener(mapButtonClickListener);

        dao = new LocationDatabase(this);
        // Applicationが処理中に応答しなくなるのを防ぐ
        new AsyncTask<Void, Void, Cursor>() {
            @Override
            protected Cursor doInBackground(Void... params) {
                // データの削除をバックグラウンドで実行する
                // UIスレッドで行うとアプリケーションの応答性が低下する
                return dao.fetchAll();
            }

            @Override
            protected void onPostExecute(Cursor cursor) {
                // データの表示はUIスレッドで行わなければならない
                startManagingCursor(cursor);
                MyAdapter temp = new MyAdapter(self, cursor);
                setListAdapter(temp);
                adapter = temp;
            }
        }.execute();
    }

    @Override
    protected void onPause() {
        unregisterReceiver(receiver);
        super.onPause();
    }

    @Override
    protected void onResume() {
        if (adapter != null)
            adapter.getCursor().requery();
        IntentFilter filter = new IntentFilter(GpsLogService.ACTION_LOCATION_CHANGED);
        registerReceiver(receiver, filter);
        super.onResume();
    }

    private synchronized String formatTime(long time) {
        date.setTime(time);
        return timeFormatter.format(date);
    }
}