package com.example.android.gpslogger;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.res.Resources;
import android.database.Cursor;
import android.database.CursorWrapper;
import android.graphics.drawable.Drawable;
import android.location.Address;
import android.location.Geocoder;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.TextView;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;
import com.google.android.maps.OverlayItem;

public class MyMapActivity extends MapActivity {

    /**
     * 記録したログを表示するためのオーバーレイです。
     * 
     */
    private final class GpsLogOverlay extends ItemizedOverlay<OverlayItem> {

        public GpsLogOverlay(Drawable balloon) {
            super(boundCenter(balloon));
        }

        @Override
        public int size() {
            if (cursor == null || cursor.isClosed())
                return 0;
            return cursor.getCount();
        }

        @Override
        protected OverlayItem createItem(int i) {
            cursor.moveToPosition(i);
            double latitude = cursor.getDouble(latitudeIndex);
            double longitude = cursor.getDouble(longitudeIndex);
            GeoPoint geoPoint = createGeoPoint(latitude, longitude);
            OverlayItem item = new OverlayItem(geoPoint, null, null);
            return item;
        }

        @Override
        protected boolean onTap(int index) {
            cursor.moveToPosition(index);
            final double latitude = cursor.getDouble(latitudeIndex);
            final double longitude = cursor.getDouble(longitudeIndex);
            final long time = cursor.getLong(timeIndex);

            final Timer timer = new Timer();
            final ProgressDialog progress = new ProgressDialog(self);

            // 座標から住所をジオコーディングするための非同期タスクを実行します。
            final AsyncTask<Void, Void, List<Address>> geocodingTask;
            geocodingTask = new AsyncTask<Void, Void, List<Address>>() {
                @Override
                protected List<Address> doInBackground(Void... params) {
                    try {
                        // getFromLocationメソッドを呼び出して、座標から住所を取得します。
                        return geocoder.getFromLocation(latitude, longitude, 1);
                    } catch (IOException e) {
                        return null;
                    }
                }

                @Override
                protected void onPostExecute(List<Address> result) {
                    // タイマーをキャンセルします。
                    timer.cancel();
                    // プログレスダイアログをキャンセルします。
                    progress.cancel();

                    // ダイアログを表示する際のバンドルに位置情報を設定します。
                    Bundle bundle = new Bundle();
                    bundle.putDouble(EX_LATITUDE, latitude);
                    bundle.putDouble(EX_LONGITUDE, longitude);
                    bundle.putLong(EX_TIME, time);
                    if (result != null && result.size() > 0) {
                        Address address = result.get(0);
                        bundle.putParcelable(EX_ADDRESS, address);
                    }

                    // ジオコーディングが終了したら、結果をダイアログに表示します。
                    showDialog(DIALOG_GEOCODE, bundle);
                }
            };
            geocodingTask.execute();

            // 一定時間(200ミリ秒)でジオコーディングが完了しなかった場合に、プログレスダイアログを表示します。
            TimerTask progressTask = new TimerTask() {
                @Override
                public void run() {
                    progress.setMessage(getString(R.string.progress_message));
                    progress.setCancelable(true);

                    // プログレスダイアログがキャンセルされたらジオコーディングを中止します。
                    DialogInterface.OnDismissListener listener;
                    listener = new DialogInterface.OnDismissListener() {
                        public void onDismiss(DialogInterface dialog) {
                            geocodingTask.cancel(false);
                        }
                    };
                    progress.setOnDismissListener(listener);

                    // ダイアログの表示処理をUIスレッドで実行します。
                    runOnUiThread(new Runnable() {
                        public void run() {
                            progress.show();
                        }
                    });
                }
            };
            timer.schedule(progressTask, 200);

            return true;
        }

        void notifyFetchComplete() {
            populate();
        }
    }
    
    
    /**
     * カーソルをラップしてrequeryが呼ばれた後にオーバーレイをpopulateするためのクラスです。
     * 
     */
    private static final class MyCursorWrapper extends CursorWrapper {
        private GpsLogOverlay overlay;

        public MyCursorWrapper(Cursor cursor, GpsLogOverlay overlay) {
            super(cursor);
            this.overlay = overlay;
        }

        @Override
        public boolean requery() {
            boolean succeed = super.requery();
            if (succeed)
                overlay.notifyFetchComplete();
            return succeed;
        }
    }


    /* ジオコーディングの結果を表示するためのダイアログのIDです。 */
    private static final int DIALOG_GEOCODE = 0;

    /* ダイアログに表示するアドレスを格納するキーです。 */
    private static final String EX_ADDRESS = "address";
    /* ダイアログに表示する緯度を格納するキーです。 */
    private static final String EX_LATITUDE = "latitude";
    /* ダイアログに表示する経度を格納するキーです。 */
    private static final String EX_LONGITUDE = "longitude";
    /* ダイアログに表示する時刻を格納するキーです。 */
    private static final String EX_TIME = "time";

    private Context self;
    private LayoutInflater layoutInflater;

    /* ログの検索結果です。 */
    private volatile Cursor cursor;

    /* ログを管理するためのデータベースです。 */
    private LocationDatabase db;

    /* ジオコーディングオブジェクトです。 */
    private Geocoder geocoder;

    /* ログの検索結果から緯度を取り出すためのインデックスです。 */
    private int latitudeIndex;
    /* ログの検索結果から経度を取り出すためのインデックスです。 */
    private int longitudeIndex;
    /* ログの検索結果から時刻を取り出すためのインデックスです。 */
    private int timeIndex;

    /* MapViewのオブジェクトです。 */
    private MapView map;
    /* 自分の位置とコンパスを表示するためのオーバーレイです。 */
    private MyLocationOverlay myLocationOverlay;
    /* ログを表示するためのオーバーレイです。 */
    private GpsLogOverlay gpsLogOverlay;

    /* 時刻のフォーマッターです。 */
    private SimpleDateFormat timeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");

    /* ログの更新通知を受け取るためのレシーバーです。 */
    private final BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            // ログの更新通知をうけとったら、再検索して最新のログを取得します。
            if (cursor != null)
                cursor.requery();
        }
    };

    /**
     * 緯度と経度からGeoPointオブジェクトを生成します。
     * 
     * @param latitude
     * @param longitude
     * @return
     */
    private static GeoPoint createGeoPoint(double latitude, double longitude) {
        // GeoPointに指定する緯度、経度は、通常の緯度、経度を100万倍したものを使います。
        int latitudeE6 = (int) (latitude * 1E6);
        int longitudeE6 = (int) (longitude * 1E6);
        GeoPoint geopoint = new GeoPoint(latitudeE6, longitudeE6);
        return geopoint;
    }

    /**
     * ログを検索します。
     */
    private void fetchLogs() {
        new AsyncTask<Void, Void, Cursor>() {

            @Override
            protected Cursor doInBackground(Void... params) {
                Cursor result = db.fetchAll();

                // カーソルから緯度、経度、時刻を取得するためのインデックスをキャッシュします。
                latitudeIndex = result.getColumnIndex("latitude");
                longitudeIndex = result.getColumnIndex("longitude");
                timeIndex = result.getColumnIndex("time");

                //初期表示の中心を決定します。
                Runnable taskForFirstFix;
                if (result.moveToLast()) {
                    // ログが記録されている場合は、最後に記録された位置を中心に初期表示します。
                    double latitude = result.getDouble(latitudeIndex);
                    double longitude = result.getDouble(longitudeIndex);
                    final GeoPoint lastPoint = createGeoPoint(latitude, longitude);
                    taskForFirstFix = new Runnable() {
                        public void run() {
                            map.getController().animateTo(lastPoint);
                        }
                    };

                } else {
                    // ログが記録されていない場合は、現在位置を中心に初期表示します。
                    taskForFirstFix = new Runnable() {
                        public void run() {
                            map.getController().animateTo(myLocationOverlay.getMyLocation());
                        }
                    };
                }
                myLocationOverlay.runOnFirstFix(taskForFirstFix);
                return result;
            }

            @Override
            protected void onPostExecute(Cursor result) {
                cursor = new MyCursorWrapper(result, gpsLogOverlay);
                // MyItemizedOverlayのnotifyFetchComplete()メソッドを呼び出して、検索の終了を通知します。
                gpsLogOverlay.notifyFetchComplete();
            };
        }.execute();
    }
    
    /**
     * ログを再検索します。
     */
    private void refetchLogs() {
        new AsyncTask<Void, Void, Void>() {
            @Override
            protected Void doInBackground(Void... params) {
                cursor.requery();
                return null;
            }
        }.execute();
    }


    /**
     * アドレスを書式化します。
     * 
     * @param address
     * @return
     */
    private String representOf(Address address) {
        if (address == null)
            return "";
        StringBuilder b = new StringBuilder();
        int lines = address.getMaxAddressLineIndex();
        for (int i = 0; i <= lines; ++i) {
            if (i > 0)
                b.append(' ');
            b.append(address.getAddressLine(i));
        }
        return b.toString();
    }

    /*
     * MapActivityのライフサイクルメソッド。
     */
    @Override
    protected boolean isRouteDisplayed() {
        return false;
    }

    /*
     * Activityのライフサイクルメソッド。アクティビティが開始する時にフレームワークが呼び出します。
     */
    @Override
    protected void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.map);
        self = this;

        layoutInflater = (LayoutInflater) getSystemService(LAYOUT_INFLATER_SERVICE);
        geocoder = new Geocoder(this);
        db = new LocationDatabase(this);

        map = (MapView) findViewById(R.id.map);
        // ズームコントロールを有効にします。
        map.setBuiltInZoomControls(true);

        // Google APIsにバンドルされているMyLocationOverlayを生成します。
        myLocationOverlay = new MyLocationOverlay(this, map);
        map.getOverlays().add(myLocationOverlay);

        // デフォルトのマーカーを指定してGPSで取得したログを表示するオーバーレイを生成します。
        Resources resources = getResources();
        gpsLogOverlay = new GpsLogOverlay(resources.getDrawable(R.drawable.green_dot));
        map.getOverlays().add(0, gpsLogOverlay);

        // MapViewのinvalidateメソッドを呼び出して、オーバーレイを追加したことを通知します。
        map.invalidate();
    }

    /*
     * Activityのライフサイクルメソッド。アクティビティが終了する時にフレームワークが呼び出します。
     */
    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (cursor != null)
            cursor.close();
        db.close();
    }

    /*
     * Activityのライフサイクルメソッド。アクティビティが前面になる時にフレームワークが呼び出します。
     */
    @Override
    protected void onResume() {
        super.onResume();

        // MyLocationOverlayのコンパスと現在位置の表示を有効化します。
        myLocationOverlay.enableCompass();
        myLocationOverlay.enableMyLocation();

        // ログを検索します。
        if (cursor == null)
            fetchLogs(); // 一度も検索が行われていない場合は、検索します。
        else
            refetchLogs(); // 一度検索が行われた場合は、再検索します。

        // ログの更新通知を受け取るためにレシーバーを登録します。
        IntentFilter filter = new IntentFilter(GpsLogService.ACTION_LOCATION_CHANGED);
        registerReceiver(receiver, filter);
    }

    /*
     * Activityのライフサイクルメソッド。アクティビティが前面でなくなる時にフレームワークが呼び出します。
     */
    @Override
    protected void onPause() {
        super.onPause();

        // MyLocationOverlayのコンパスと現在位置の表示を無効化します。
        myLocationOverlay.disableCompass();
        myLocationOverlay.disableMyLocation();

        // カーソルを非活性化します。
        if (cursor != null)
            cursor.deactivate();

        // ログの更新通知の受け取りを停止します。
        unregisterReceiver(receiver);
    }

    /*
     * Activityのライフサイクルメソッド。ダイアログが最初に表示される時にフレームワークが呼び出します。
     */
    @Override
    protected Dialog onCreateDialog(int id, Bundle args) {
        switch (id) {
        case DIALOG_GEOCODE:
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            View v = layoutInflater.inflate(R.layout.geocode, null);
            builder.setView(v);
            return builder.create();
        default:
            return super.onCreateDialog(id, args);
        }
    }

    /*
     * Activityのライフサイクルメソッド。ダイアログが表示される時に毎回、フレームワークが呼び出します。
     */
    @Override
    protected void onPrepareDialog(int id, Dialog dialog, Bundle args) {
        switch (id) {
        case DIALOG_GEOCODE:
            String latitude = String.valueOf(args.getDouble(EX_LATITUDE));
            String longitude = String.valueOf(args.getDouble(EX_LONGITUDE));
            String time = timeFormat.format(new Date(args.getLong(EX_TIME)));
            String address = representOf((Address) args.getParcelable(EX_ADDRESS));

            ((TextView) dialog.findViewById(R.id.latitude)).setText(latitude);
            ((TextView) dialog.findViewById(R.id.longitude)).setText(longitude);
            ((TextView) dialog.findViewById(R.id.time)).setText(time);
            ((TextView) dialog.findViewById(R.id.address)).setText(address);
            break;
        default:
            super.onPrepareDialog(id, dialog, args);
            break;
        }
    }
}
